package clients

import (
	"context"
	"fmt"
	trashlogProto "gitea.pena/PenaSide/trashlog/proto/generated"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
)

type TrashLogClient struct {
	conn   *grpc.ClientConn
	client trashlogProto.TrashlogClient
}

func NewTrashLogClient(address string) (*TrashLogClient, error) {
	conn, err := grpc.Dial(address, grpc.WithTransportCredentials(insecure.NewCredentials()))
	if err != nil {
		return nil, fmt.Errorf("failed to connect to RPC server: %v", err)
	}

	client := trashlogProto.NewTrashlogClient(conn)
	return &TrashLogClient{conn: conn, client: client}, nil
}

func (c *TrashLogClient) Ping(ctx context.Context) error {
	_, err := c.client.Ping(ctx, &trashlogProto.Dummy{})
	if err != nil {
		return fmt.Errorf("RPC ping failed: %v", err)
	}
	return nil
}

func (c *TrashLogClient) Close() {
	if c.conn != nil {
		c.conn.Close()
	}
}
