package zaptrashlog_test

import (
	"context"
	"database/sql"
	"fmt"
	"gitea.pena/PenaSide/trashlog/app"
	"github.com/skeris/appInit"
	bolt "go.etcd.io/bbolt"
	"sync"
	"testing"

	"gitea.pena/PenaSide/trashlog/wrappers/zaptrashlog"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"gitea.pena/PenaSide/hlog"
	//bolt "go.etcd.io/bbolt"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
	"time"
)

var _ = Describe("Zaptrashlog", func() {
	logger, err := zap.NewProduction()

	if err != nil {
		panic(err)
	}
	logger = logger.Named("test")

	ctx := context.Background()
	fmt.Println("MMMMM")
	a, err := app.New(ctx, app.Options{
		Development:   true,
		AppName:       "trashlog-test",
		GrpcURL:       ":7112",
		ClickhouseURL: "tcp://127.0.0.1:9000?debug=true",
	}, appInit.Version{
		Release:   "test",
		Commit:    "",
		BuildTime: "",
	})
	if err != nil {
		panic(err)
	}
	a.GetErr()

	tl, err := zaptrashlog.NewCore(
		ctx,
		zap.InfoLevel,
		"localhost:7112",
		"v0",
		"test",
		0,
	)
	if err != nil {
		fmt.Println("Atatat", err, *tl)
	}

	logger = logger.WithOptions(zap.WrapCore(func(core zapcore.Core) zapcore.Core {
		return zapcore.NewTee(core, tl)
	}))
	log := hlog.New(logger).Module("test1")

	Context("Recovering", func() {
		It("SendSomeData", func() {
			log.Emit(InfoTest{Test: "test123"})
			time.Sleep(time.Second * 10)
			Expect(tl.Close()).To(BeNil())
			db, err := bolt.Open("recover.bolt", 0666, nil)
			Expect(err).To(BeNil())
			Expect(db).ToNot(BeNil())
			defer db.Close()

			fmt.Println("Ora!", "Ora!", "Ora!")
			Expect(db.View(func(tx *bolt.Tx) error {
				b := tx.Bucket([]byte("recover"))

				fmt.Println("Ora!", b.Stats().KeyN)

				if err := b.ForEach(func(k, v []byte) error {

					fmt.Println("Ora!", k, v)
					return fmt.Errorf("%s", v)
				}); err != nil {
					return err
				}
				return nil
			})).ToNot(BeNil())
		})
	})

	Context("Main", func() {
		time.Sleep(time.Second * 5)
		It("LogSomeData", func() {
			connect, err := sql.Open(
				"clickhouse",
				"tcp://127.0.0.1:9000?debug=true",
			)
			if err != nil {
				return
			}

			row := connect.QueryRowContext(ctx, `SELECT count(1) from statistics`)
			var col, col2 int
			Expect(row.Scan(&col)).To(BeNil())
			log.Emit(InfoTest{Test: "test"})
			time.Sleep(time.Second * 3)
			row = connect.QueryRowContext(ctx, `SELECT count(1) from statistics`)
			Expect(row.Scan(&col2)).To(BeNil())
			Expect(col).ToNot(Equal(col2))
		})
	})
})

type InfoTest struct {
	Test string
}

type InfoLogger1 struct {
	Name string
}

type InfoLogger2 struct {
	Name string
}

// тест по сути ничего не проверяет просто два логгера которые емитят, ожидалась паника
// так как мьютекс не был указан в with, но паники не было
func TestMutex(t *testing.T) {
	ctx1 := context.Background()
	ctx2 := context.Background()

	logger1, err := zap.NewProduction()
	if err != nil {
		panic(err)
	}
	logger2, err := zap.NewProduction()
	if err != nil {
		panic(err)
	}

	clickHouseLogger1, err := zaptrashlog.NewCore(ctx1, zap.InfoLevel, "localhost:7113", "1", "1", time.Now().Unix())
	if err != nil {
		panic(err)
	}

	clickHouseLogger2, err := zaptrashlog.NewCore(ctx2, zap.InfoLevel, "localhost:7113", "2", "2", time.Now().Unix())
	if err != nil {
		panic(err)
	}

	loggerForHlog1 := logger1.WithOptions(zap.WrapCore(func(core zapcore.Core) zapcore.Core {
		return zapcore.NewTee(core, clickHouseLogger1)
	}))

	loggerHlog1 := hlog.New(loggerForHlog1).Module("1")

	loggerForHlog2 := logger2.WithOptions(zap.WrapCore(func(core zapcore.Core) zapcore.Core {
		return zapcore.NewTee(core, clickHouseLogger2)
	}))

	loggerHlog2 := hlog.New(loggerForHlog2).Module("2")

	loggerHlog1.With(InfoTest{})
	loggerHlog2.With(InfoTest{})
	go func() {
		count1 := 1
		for {
			level := fmt.Sprintf("loggerHlog1-Number%d", count1)
			loggerHlog1.Emit(InfoTest{Test: level})
			count1++
			time.Sleep(time.Second * 1)
		}
	}()

	go func() {
		count2 := 1
		for {
			level := fmt.Sprintf("loggerHlog2-Number%d", count2)
			loggerHlog2.Emit(InfoTest{Test: level})
			count2++
			time.Sleep(time.Second * 1)
		}
	}()

	loggerHlog1.With(InfoLogger1{})
	loggerHlog2.With(InfoLogger2{})

	go func() {
		count1 := 1
		for {
			level := fmt.Sprintf("InfoLogger1-Number%d", count1)
			loggerHlog1.Emit(InfoLogger1{Name: level})
			count1++
			time.Sleep(time.Second * 1)
		}
	}()

	go func() {
		count2 := 1
		for {
			level := fmt.Sprintf("InfoLogger2-Number%d", count2)
			loggerHlog2.Emit(InfoLogger2{Name: level})
			count2++
			time.Sleep(time.Second * 1)
		}
	}()

	time.Sleep(time.Second * 10)
}

//тест чекает на пересоздание мьютекса в методе write, суть проблемы в том что это может порождать несколько мьютексов
//в тесте должны получить то что один мьютекс используется даже после пересоздания его в методе with
// алгоритм таков: создается в ядре -> потом создается экземпляр от zap и hlog  -> проверка на то что senfmutex один,
// проверка средствами trylock, после emit вызывается write в котором лочим мьютекс и не разлочиваем, далее чекаем на то что
// основной логер от ядра который родитель этого мьютекса тоже имеет заблокированный мьютекс, это доказывает то что
//мьютекс один даже после with метода

func TestSendMutex(t *testing.T) {
	l, err := zap.NewDevelopment()
	if err != nil {
		panic(err)
	}
	logger1, err := zaptrashlog.NewCore(context.Background(), zapcore.InfoLevel, "localhost:7113", "1", "commit123", time.Now().Unix())
	if err != nil {
		panic(err)
	}

	loggerForHlog1 := l.WithOptions(zap.WrapCore(func(core zapcore.Core) zapcore.Core {
		return zapcore.NewTee(core, logger1)
	}))

	loggerHlog1 := hlog.New(loggerForHlog1).Module("loggerForHlog1")
	llooggerZap := loggerHlog1.With(InfoLogger1{})

	var wg sync.WaitGroup
	wg.Add(1)

	go func() {
		defer wg.Done()
		llooggerZap.Emit(InfoTest{Test: "test"}) // мьютекс лочим
	}()

	time.Sleep(1 * time.Second)

	// TryLock в логгере корневом что он заблокирован
	resultLock := logger1.SendMutex.TryLock()
	if resultLock {
		fmt.Println(resultLock)
		t.Error("Должен быть заблокирован но это не так")
	} else {
		fmt.Println(resultLock)
		t.Log("sendMutex заблокирован ок")
	}

	wg.Wait()
}
